#version 460 core

#include "/lib/settings.glsl"
#include "/lib/lightmap.glsl"

uniform int isEyeInWater;
uniform float blindness, far, rainStrength;
uniform vec3 chunkOffset, shadowLightPosition;
uniform vec4 entityColor;
uniform mat3 normalMatrix;
uniform mat4 modelViewMatrix, projectionMatrix;

in vec2 vaUV0;
in vec3 vaNormal, vaPosition;
in vec4 vaColor;

out float depth, diffuse, fog;
out vec2 texCoord;
out vec4 tintColor;

#ifdef SHADOWS
	uniform mat4 gbufferModelViewInverse, shadowModelView, shadowProjection;

	out float bias;
	out vec3 shadowPos;

	#include "/lib/distort.glsl"
#endif

void main() {
	texCoord = vaUV0;

	vec4 viewPos = modelViewMatrix * vec4(vaPosition + chunkOffset, 1.0);

	#ifdef SHADOWS
		vec3 rawShadowPos = (shadowProjection * shadowModelView * gbufferModelViewInverse * viewPos).xyz;

		bias = max(0.001 * length(rawShadowPos.xy), 0.00005);

		shadowPos = distort(rawShadowPos) * 0.5 + 0.5;
	#endif

	fog = min(pow(length(viewPos.xyz) / far, 10.0 / (FOG + blindness * 5000.0 + rainStrength * RAIN_FOG + pow(isEyeInWater, 4.0) * WATER_FOG)), 1.0);

	vec4 position = projectionMatrix * viewPos;

	depth = position.z / position.w;

	diffuse = dot(normalize(normalMatrix * vaNormal), normalize(shadowLightPosition));

	tintColor = vec4(mix(vaColor.rgb, entityColor.rgb, entityColor.a), 1.0);

	vec3 light = texture(lightmap, lmCoord).rgb;

	#ifdef END
		light = mix(vec3(0.76, 0.4, 0.76) * AMBIENT * 0.05, light * 1.5, float(light) - 0.75);
	#endif

	tintColor.rgb *= light;

	gl_Position = position;
}